<?php
/**
 * @package     plg_easyshop_related
 * @version     1.1.0
 * @Author      www.joomtech.net
 * @copyright   Copyright (C) 2015 - 2018 www.joomtech.net. All Rights Reserved.
 * @license     GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */

defined('_JEXEC') or die;
\JLoader::import('easyshop.plugin.addon');

use ES\Plugin\PluginAddon;
use Joomla\Utilities\ArrayHelper;

class PlgEasyshopRelated extends PluginAddon
{
	protected $addons = ['product'];
	protected $addonIcon = 'compress';

	protected function getAssociations($itemId)
	{
		$itemId = (int) $itemId;
		$db     = $this->db;
		$query  = $db->getQuery(true)
			->select('DISTINCT a.item_id')
			->from($db->quoteName('#__easyshop_related_associations', 'a'))
			->where('a.item_id <> ' . $itemId)
			->where('a.ref_ids LIKE ' . $db->quote('%[' . $itemId . ']%'));
		$db->setQuery($query);

		return $db->loadColumn();
	}

	public function onProductAddonDisplay($product)
	{
		$params = $this->addonClass->getData('related.product', $product->id);
		$pks    = $params->get('related_ids', []);

		if ($this->params->get('associations', '0'))
		{
			if ($refIds = $this->getAssociations($product->id))
			{
				$pks = array_merge($pks, $refIds);
			}
		}

		if ($this->params->get('relatedByCategory', '1'))
		{
			$query = $this->db->getQuery(true)
				->select('a.id')
				->from($this->db->quoteName('#__easyshop_products', 'a'))
				->where('a.id <> ' . (int) $product->id)
				->where('a.category_id = ' . (int) $product->category_id);
			$this->db->setQuery($query);

			if ($newIds = $this->db->loadColumn())
			{
				$pks = array_merge($pks, $newIds);
			}
		}

		if ($this->params->get('relatedByTag', '1'))
		{
			$query = $this->db->getQuery(true)
				->select('a.item_id')
				->from($this->db->quoteName('#__easyshop_tag_items', 'a'))
				->join('INNER', $this->db->quoteName('#__easyshop_tags', 'a2') . ' ON a2.id = a.tag_id')
				->where('a2.context = ' . $this->db->quote('com_easyshop.product'))
				->where('a.item_id <> ' . (int) $product->id);
			$this->db->setQuery($query);

			if ($newIds = $this->db->loadColumn())
			{
				$pks = array_merge($pks, $newIds);
			}
		}

		if (!empty($pks))
		{
			/** @var $productClass \ES\Product */
			$productClass = easyshop('class.product');
			$pks          = ArrayHelper::toInteger($pks);
			$pks          = ArrayHelper::arrayUnique($pks);
			$pks          = array_values($pks);
			$renderer     = $this->getRenderer();
			$products     = [];

			foreach ($pks as $pk)
			{
				if ($pk == $product->id
					|| !$productClass->isEnabled($pk, false)
				)
				{
					continue;
				}

				if ($relatedProduct = $productClass->getItem($pk))
				{
					$relatedProduct->params->merge($params);
					$products[] = $relatedProduct;
				}
			}

			$product->relatedBuffer = $renderer->render('related.related', [
				'params'   => $this->params,
				'products' => $products,
				'config'   => $this->getConfig(),
			]);
		}
		else
		{
			$product->relatedBuffer = '';
		}

		parent::onProductAddonDisplay($product);
	}

	public function onEasyshopAddonAfterSave($context, $addons, $itemId)
	{
		$params = $this->addonClass->getData('related.product', $itemId);
		$refIds = $params->get('related_ids', []);

		if ($context != 'com_easyshop.product'
			|| empty($addons['related'])
			|| empty($refIds)
		)
		{
			return;
		}

		$refIds[] = $itemId;
		$refIds   = ArrayHelper::toInteger($refIds);
		$refIds   = ArrayHelper::arrayUnique($refIds);
		$db       = $this->db;
		$query    = $db->getQuery(true)
			->delete($db->qn('#__easyshop_related_associations'))
			->where($db->qn('item_id') . ' = ' . $itemId);
		$db->setQuery($query)
			->execute();
		$query->clear()
			->insert($db->qn('#__easyshop_related_associations'))
			->columns(['item_id', 'ref_ids'])
			->values($itemId . ',' . $db->q('[' . join('][', $refIds) . ']'));
		$db->setQuery($query)
			->execute();
	}

	public function onEasyshopAddonPrepareForm(JForm $form, $data)
	{
		parent::onEasyshopAddonPrepareForm($form, $data);

		if ($form->getName() == 'plg_easyshop_related.addon.product')
		{
			$productId = (int) $this->app->input->getInt('id', 0);

			if ($productId > 0)
			{
				if (empty($data->related_ids))
				{
					$data->related_ids = [];
				}

				if ($this->params->get('associations', '0'))
				{
					if ($refIds = $this->getAssociations($productId))
					{
						$data->related_ids = array_merge($data->related_ids, $refIds);
						$data->related_ids = ArrayHelper::arrayUnique($data->related_ids);
						$data->related_ids = array_values($data->related_ids);
					}
				}

				foreach ($data->related_ids as &$id)
				{
					if ((int) $id === $productId)
					{
						unset($id);
						break;
					}
				}
			}
		}
	}
}
